{% load staticfiles %}
{% load i18n %}
<div class='mojeid_registration'>
    <a href='{% url 'openid-registration' attribute_set=attribute_set %}'><img  src={% static "mojeid/zalozit_green_3D.png" %} alt='{% trans "Create a mojeID account" %}' /></a>
</div>
