<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head></head>
<body onload='window.document.forms[0].submit();'>
<form action='{{ action }}' method='post' style='display:none'>
    <input type='hidden' name='realm' value='{{ realm }}' />
    <input type='hidden' name='registration_nonce' value='{{ nonce }}' />
{% for field in fields %}
        {{ field|safe }}
{% endfor %}
    <input type='submit' value='submit' />
</form>
</body>
</html>
