{% load staticfiles %}
{% load i18n %}
{% get_current_language as LANGUAGE_CODE %}
<div class='mojeid_login'>
    <form action='{% url 'openid-init' attribute_set=attribute_set %}' method='post'>
        {% csrf_token %}
        <input type="hidden" name="next" value="{% if redirect %}{{ redirect }}{% else %}{{ request.get_full_path }}{% endif %}">
        <input type="hidden" name="lang" value="{{ LANGUAGE_CODE }}">
        <table>
            <tr>
                <th colspan="2">
                    {% if LANGUAGE_CODE == "en" %}
                    <input type="image"  src="{% static "mojeid/black_162x32_en.png" %}">
                    {% else %}
                    <input type="image"  src="{% static "mojeid/black_162x32.png" %}">
                    {% endif %}
                </th>
            </tr>
            <tr>
                <td>
                    <a href='{% if why_my_id_url %}{{ why_my_id_url }}{% else %}http://www.mojeid.cz/page/805/vyhody-mojeid/{% endif %}'>{% trans "Why mojeID?" %}</a>
                </td>
                <td>
                    <a href='{% url 'openid-registration' %}'>{% trans "Create a mojeID account" %}</a>
                </td>
            </tr>
        </table>
    </form>
</div>
